/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2004, 2020                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or disclosure restricted by GSA ADP Schedule Contract        */
/*      with IBM Corp.                                               */
/******************************************************************* */
/*--START OF APAR HISTORY------------------------------------------- */
/*  APAR    DATE      DESCRIPTION                                    */
/*  @FIX-1  20091022  Correct case where the last line in the from   */
/*                    program is not appearing in the converted      */
/*                    program.                                       */
/*  @FIX-2  20200319  Correct segmentation fault when an input file  */
/*                    has blank lines.  Also, added diagnotic        */
/*                    printf()s to help understand problems. These   */
/*                    diagnostic printf()s are commented out.        */
/*                                                                   */
/*--END OF APAR HISTORY--------------------------------------------- */
/*********************************************************************/

#include <stdio.h>
#include <string.h>                                        /* @FIX-1 */
#define  MAXLINE  80

int   get_source_line(char *, FILE *);
int   is_branch_to_label(int, int, char *);
int   is_bal_to_label(int, int, char *);
int   convert_shift_inst(int, int, char *, char *);
int   convert_halfword(int, int, char *, char *, int);

/*********************************************************************/
/*                                                                   */
/*  Convert assembler programs to use 64-bit instructions.           */
/*                                                                   */
/*  required format:  <command> input_file output_file               */
/*  optional format:  <command> input_file output_file -br -reg -mac */
/*                                                     -half -half31 */
/*                                                     -sidcode xxx  */
/*                                                     -verify       */
/*                                                     -spm          */
/*                                                                   */
/*  The intent of this program is to take an assembler program and   */
/*  change register usage so that all 64-bits of a register are      */
/*  properly initialized.  This will allow the assembler program to  */
/*  run in 64-bit mode.  For example:                                */
/*                                                                   */
/*       MI0MI REG=R1                                                */
/*       L     R1,CE1CR0     Get base of input message               */
/*       CLC   MI0ACC(5),=C'ZDSYS' Check input message               */
/*                                                                   */
/*  These instructions will work perfectly in 31-bit mode.  They     */
/*  may not work in 64-bit mode.  In 64-bit mode the Load (L)        */
/*  instruction will put the value at CE1CR0 into the low order 4    */
/*  bytes of R1.  The high order 4 bytes are unaffected.  The Compare*/
/*  logical (CLC) instruction will use all use R1 as a base and it   */
/*  use all 8-bytes of the register to calculate the address.  If the*/
/*  high order 4-bytes is anything other than 0, the CLC will not    */
/*  get the correct address to compare.                              */
/*                                                                   */
/*  When -br is specified, branch instructions will be converted.    */
/*  When -reg is specified, register instructions will be converted. */
/*  When -mac is specified, specific macros will be converted        */
/*  When none of -br -reg -half are specified, the default will be   */
/*  as though -br, -imm, and -reg are specified.  However, if any    */
/*  option is specified, the defaults will be overridden and only    */
/*  the specified option will be done.  For example, if only -br is  */
/*  specified, only the branch instructions will be converted.       */
/*                                                                   */
/*  Register instructions being converted are:   (-reg)              */
/*                                                                   */
/*  AR   -> AGR      ALR  -> ALGR     AHI  -> AGHI     A    -> AGF   */
/*  AL   -> ALGF                                                     */
/*  BCT  -> BRCTG    BXH  -> BXHG     BCTR -> BCTGR    BXLE -> BXLEG */
/*  CR   -> CGR      CLR  -> CLGR     CHI  -> CGHI     CL   -> CLGF  */
/*  C    -> CGF  (if 2nd operand is not a literal)                   */
/*  C    -> CLGF (if 2nd operand is a literal)                       */
/*  L    -> LLGF     LR   -> LGR      LH   -> LGH      LCR  -> LCGR  */
/*  LNR  -> LNGR     LPR  -> LPGR     LTR  -> LTGR     LHI  -> LGHI  */
/*  MHI  -> MGHI                                                     */
/*  NR   -> LGR                                                      */
/*  OR   -> OGR                                                      */
/*  SR   -> SGR      SLA  -> SLAG     SLL  -> SLLG     SLR  -> SLGR  */
/*  SRA  -> SRAG     SRL  -> SRLG     S    -> SGF      SL   -> SLGF  */
/*  XR   -> XGR                                                      */
/*                                                                   */
/*  ...WARNING about converting L to LLGF...                         */
/*  ...LLGF will put a 4-byte value from the storage location into   */
/*  ...the low order 32-bits of the register and zero the high order */
/*  ...32-bits.  Other choices for L are LLGT and LGF.  LLGT will    */
/*  ...take the low order 31-bits from the storae location and put it*/
/*  ...into the low order 31-bits of the regsiter and zero the high  */
/*  ...order 33-bits.  This is useful for putting linkages (BAS R14,.*/
/*  ...into a 64-bit register.  LGF will put a 4-byte value from the */
/*  ...storage location into the low order 32-bits of the register   */
/*  ...and sign extend the value for the high order 32-bits.         */
/*                                                                   */
/*                                                                   */
/*  ...WARNING about targets of EX (execute) instruction...          */
/*  ...This program may convert the target of an EX instruction.     */
/*  ...This may cause the execution of the program to be different   */
/*  ...than is anticipated before conversion.  For example, if the   */
/*  ...following code exists in your segment:                        */
/*             EX    R1,TARGET                                       */
/*     TARGET  LR    R0,R0                                           */
/*  ...it would be converted to:                                     */
/*             EX    R1,TARGET                                       */
/*     TARGET  LGR   R0,R0                                           */
/*  ...In this case, the LGR may not be desired.  Targets of EX      */
/*  ...(execute) need to be investigated to determine if conversion  */
/*  ...is desired and if after conversion it is operating properly.  */
/*                                                                   */
/*                                                                   */
/*  ...WARNING about ICM...                                          */
/*  ...If the following is coded:                                    */
/*             ICM   R3,B'1111',LABEL                                */
/*  ...the conversion program will change the code to be:            */
/*             LLGF  R3,LABEL                                        */
/*             LTGR  R3,R3                                           */
/*  ...if a branch follows the ICM and the condition being checked   */
/*  ...is anything other than BZ or BNZ, this conversion will not    */
/*  ...work.  If the branch is not BZ or BNZ, the progammer should   */
/*  ...change what this conversion program did.                      */
/*                                                                   */
/*  ...WARNING about LM...                                           */
/*  ...The conversion program will not do anything with load         */
/*  ...mulitple (LM).  This means that only the low order 32-bits    */
/*  ...of the registers will be initialized.  If the register had a  */
/*  ...value in the high order 4 bytes, it would remain there and    */
/*  ...if this register is used as a base register in 64-bit mode,   */
/*  ...the address may be wrong.                                     */
/*  ...The conversion program will mark each LM and STM instruction  */
/*  ...with a sidcode of RESEARCH.                                   */
/*                                                                   */
/*  ...WARNING about LTR...                                          */
/*  ...If the following is coded:                                    */
/*             L     R1,Label                                        */
/*             LTR   R1,R1                                           */
/*  ...the conversion program would generate the following:          */
/*             LLGF  R1,Label                                        */
/*             LTGR  R1,R1                                           */
/*  ...If the value at Label is negative, the converted instructions */
/*  ...would not get the same result.  Instead, the Load (L) should  */
/*  ...have been converted to:                                       */
/*             LGF   R1,Label                                        */
/*             LTGR  R1,R1                                           */
/*                                                                   */
/*  ...WARNING about LA...                                           */
/*  ...With option -half or -half31, LA instructions may be converted*/
/*  ...to AGHI.  The LA instruction does not change the condition    */
/*  ...code where AGHI does.  Therefore, if the following set of     */
/*  ...instructions are coded, they will not work properly after     */
/*  ...conversion:                                                   */
/*             TM    EBW000,X'80'                                    */
/*             LA    R1,4(,R1)                                       */
/*             BZ    Label                                           */
/*  ...conversion with -half option would be:                        */
/*             TM    EBW000,X'80'                                    */
/*             AGHI  R1,4        <- sets condition code              */
/*             BZ    Label                                           */
/*                                                                   */
/*  ...WARNING about SLL...                                          */
/*  ...SLL will be converted to SLLG. The SLL will cause the high    */
/*  ...order bits to be shifted out of the register.  By changing to */
/*  ...SLLG, what was the high order bits with a 32-bit register are */
/*  ...no longer the high order bits.  If the intent of the SLL in   */
/*  ...get rid of the high order bits, convering to SLL will only    */
/*  ...move the bits into the high order 4 bytes of the register.    */
/*  ...In this case, it may be best to leave the SLL instruction.    */
/*                                                                   */
/*  Branch instructions being converted are:     (-br)               */
/*                                                                   */
/*  B    -> J        BE   -> JE       BH   -> JH       BL   -> JL    */
/*  BL   -> JL       BM   -> JM       BO   -> JO       BP   -> JP    */
/*  BZ   -> JZ       BAL  -> BRAS     BAS  -> BRAS     BNE  -> JNE   */
/*  BNH  -> JNH      BNL  -> JNL      BNM  -> JNM      BNO  -> JNO   */
/*  BNP  -> JNP      BNZ  -> JNZ      BC   -> BRC                    */
/*                                                                   */
/*  ...Warning about branch relative....                             */
/*  ...If the label is not in the program, for example CPMEXR is in  */
/*  ...CAPT and a branch relative is done to it in CCMCDC, an        */
/*  ...assembly error would happen.                                  */
/*                                                                   */
/*  Macros being converted are:                  (-mac)              */
/*                                                                   */
/*  $GEVAC -> GEVAC  $GSVAC -> GSVAC  $MOVEC -> MOVEC                */
/*  $SWSPC -> SWSPC  $LOCKC -> LOCKC  $UNLKC -> UNLKC                */
/*  $ADPC  -> ADPC   $CCWBC -> CCWBC  $CCWTC -> CCWTC                */
/*  $CPUC  -> CPUC   $CRISC -> CRISC  $DCOLC -> DCOLC                */
/*  $FINDC -> FINDC  $FORKC -> FORKC  $GCFBC -> GCFBC                */
/*  $GETRC -> GETRC  $GIOBC -> GIOBC                                 */
/*  $LCKRC -> LCKRC  $MASKC -> MASKC  $RCFBC -> RCFBC                */
/*  $RECVC -> RECVC  $RELRC -> RELRC  $RETRC -> RETRC                */
/*  $RIOBC -> RIOBC  $TCPLC -> TCPLC  $ULKRC -> ULKRC                */
/*  $RSWBC -> RSWBC  $RELBC -> RELBC                                 */
/*                                                                   */
/*  Halfword instructions being converted are:   (-half)             */
/*  The intent is to reduce the number of bytes needed under a base  */
/*  register.  Instructions used for -reg are larger.  This is just  */
/*  one way to partially offset the larger instructions.             */
/*                                                                   */
/*  AH  when the second operand is a literal -> AGHI                 */
/*  CH  when the second operand is a literal -> CGHI                 */
/*  LA  Rx,number                            -> LGHI  Rx,number      */
/*  LA  Rx,number(Rx)                        -> AGHI  Rx,number      */
/*  LH  when the second operand is a literal -> LGHI                 */
/*  MH  when the second operand is a literal -> MGHI                 */
/*  SH  when the second operand is a literal -> AGHI the immediate   */
/*                                               operand is negative */
/*                                                                   */
/*  The sidcode which marks changes can be specified:  (-sidcode)    */
/*  -SIDCODE xxxxxxxx   where xxxxxxxx is the string that will be put*/
/*  starting in column 73 for all lines that are changed.  The       */
/*  sidcode can be upto 8 characters in length.  If -sidcode is not  */
/*  specified or if more than 8 characters are used, the default     */
/*  sidcode will be 64MODE.                                          */
/*                                                                   */
/*  If a line should be converted and this program determines that   */
/*  there is not enough white space to convert it, sidcode 64TODO    */
/*  will be placed starting in column 73.                            */
/*                                                                   */
/*  -verify will verify that the file extensions of the input file   */
/*  are either .asm, .cpy, or .mac.  If -verify is not specified, no */
/*  no verification will be done and any file extension can be used. */
/*                                                                   */
/*  -spm will convert structured programming macros (SPMs).  The     */
/*  following changes will be made:                                  */
/*                                                                   */
/*  #CASENTR -> #CAST        #ECASE   -> #ECAS                       */
/*  #ELOOP   -> #ELOP        #ELSEIF  -> #ELIF                       */
/*  #ESRCH   -> #EDO         #ORELSE  -> #OREL                       */
/*  #SRCEX   -> #DOEX        #STRTSRC -> #DO                         */
/*                                                                   */
/*********************************************************************/

main(int argc, char *argv[])
{
   char  c;
   char  *from_pgm;
   char  *to_pgm;
   char  operand[MAXLINE+2];
   char  operand2[MAXLINE+2];
   char  source_line[MAXLINE+2];
   char  new_source_line[MAXLINE+2];
   char  output_source_line[MAXLINE+2];
   char  *new_inst_ptr;
   FILE  *from_fp;
   FILE  *to_fp;
   int   h;             /* used as boolean                           */
   int   i;             /* used as index                             */
   int   k;             /* used as index                             */
   int   m;             /* used as index                             */
   int   x;             /* used as index                             */
   int   y;             /* used as index                             */
   int   rc;            /* used as return code from get_source_line  */
   int   comment;
   int   convert_br;
   int   convert_half;
   int   convert_half31;
   int   convert_reg;
   int   convert_macro;
   int   convert_spm;
   int   first_time_thru;
   int   label_size;
   int   lines_converted;
   int   lines_not_converted;
   int   lines_to_be_investigated;
   int   line_nbr;
   int   previous_line_continued;
   int   sign;
   int   size_of_old_inst;
   int   size_of_new_inst;
   int   start_of_icm;
   int   this_line_continued;
   int   use_sidcode;
   int   verify_input_file_extensions;

   /* Instructions */

   char agf[]   = "AGF";
   char agr[]   = "AGR";
   char ahi[]   = "AHI";
   char aghi[]  = "AGHI";
   char algf[]  = "ALGF";
   char algr[]  = "ALGR";
   char brc[]   = "BRC";
   char bctg[]  = "BCTG";
   char bctgr[] = "BCTGR";
   char bras[]  = "BRAS";
   char brct[]  = "BRCT";
   char brctg[] = "BRCTG";
   char brxhg[] = "BRXHG";
   char brxlg[] = "BRXLG";
   char cgf[]   = "CGF";
   char cgr[]   = "CGR";
   char chi[]   = "CHI";
   char cghi[]  = "CGHI";
   char clgf[]  = "CLGF";
   char clgr[]  = "CLGR";
   char j[]     = "J";
   char je[]    = "JE";
   char jh[]    = "JH";
   char jl[]    = "JL";
   char jm[]    = "JM";
   char jo[]    = "JO";
   char jp[]    = "JP";
   char jz[]    = "JZ";
   char jne[]   = "JNE";
   char jnh[]   = "JNH";
   char jnl[]   = "JNL";
   char jnm[]   = "JNM";
   char jno[]   = "JNO";
   char jnp[]   = "JNP";
   char jnz[]   = "JNZ";
   char lcgr[]  = "LCGR";
   char lgr[]   = "LGR";
   char lgh[]   = "LGH";
   char lhi[]   = "LHI";
   char lghi[]  = "LGHI";
   char llgf[]  = "LLGF";
   char lngr[]  = "LNGR";
   char lpgr[]  = "LPGR";
   char ltgr[]  = "LTGR";
   char mhi[]   = "MHI";
   char mghi[]  = "MGHI";
   char ngr[]   = "NGR";
   char ogr[]   = "OGR";
   char sgf[]   = "SGF";
   char sgr[]   = "SGR";
   char slag[]  = "SLAG";
   char slgf[]  = "SLGF";
   char slgr[]  = "SLGR";
   char sllg[]  = "SLLG";
   char srag[]  = "SRAG";
   char srlg[]  = "SRLG";
   char xgr[]   = "XGR";

   /* Macros */

   char adpc[]  = "ADPC ";
   char ccwbc[] = "CCWBC ";
   char ccwtc[] = "CCWTC ";
   char cpuc[]  = "CPUC ";
   char crisc[] = "CRISC ";
   char dcolc[] = "DCOLC ";
   char findc[] = "FINDC ";
   char forkc[] = "FORKC ";
   char gcfbc[] = "GCFBC ";
   char getrc[] = "GETRC ";
   char gevac[] = "GEVAC ";
   char giobc[] = "GIOBC ";
   char gsvac[] = "GSVAC ";
   char lckrc[] = "LCKRC ";
   char lockc[] = "LOCKC ";
   char maskc[] = "MASKC ";
   char movec[] = "MOVEC ";
   char rcfbc[] = "RCFBC ";
   char recvc[] = "RECVC ";
   char relbc[] = "RELBC ";
   char relrc[] = "RELRC ";
   char retrc[] = "RETRC ";
   char riobc[] = "RIOBC ";
   char rswbc[] = "RSWBC ";
   char swspc[] = "SWSPC ";
   char tcplc[] = "TCPLC ";
   char ulkrc[] = "ULKRC ";
   char unlkc[] = "UNLKC ";

   /* SPMs */

   char spm_cast[]  = "#CAST";
   char spm_do[]    = "#DO";
   char spm_doex[]  = "#DOEX";
   char spm_ecas[]  = "#ECAS";
   char spm_edo[]   = "#EDO";
   char spm_elif[]  = "#ELIF";
   char spm_elop[]  = "#ELOP";
   char spm_orel[]  = "#OREL";

   /* Misc */

   char default_sidcode[] = "64MODE  ";
   char sidcode[]        = "        ";
   char errorcode[]      = "64TODO  ";
   char investigatecode[]= "RESEARCH";

   /******************************************************************/
   /*                                                                */
   /*  Read in the parameters                                        */
   /*                                                                */
   /*  -i = from_pgm or program that is being converted              */
   /*  -o = to_pgm or output or newly converted program              */
   /*                                                                */
   /******************************************************************/

   if (argc < 3)
   {
      printf("Invalid number of parameters input\n");
      return(0);
   }                          /* end if argc <= 2                    */

   from_pgm = argv[1];
   to_pgm = argv[2];

   if (argc >= 4)
   {
      /* set to no conversion and let parms decide what to do        */
      convert_br = 0;         /* don't convert all branch instr      */
      convert_reg = 0;        /* don't convert all register type inst*/
      convert_macro = 0;      /* don't convert all macros            */
      convert_half = 0;       /* don't convert halfword instr        */
      convert_half31 = 0;     /* don't convert halfword instr to 31  */
      convert_spm = 0;        /* don't convert SPMs                  */
      use_sidcode = 0;        /* use default sidcode 64MODE          */
      verify_input_file_extensions = 0; /* Don't verify file ext     */

      for(i=3; i < argc; i++)
      {

         /************************************************************/
         /*                                                          */
         /* -br option ... convert all branches                      */
         /*                                                          */
         /************************************************************/

         if(argv[i][0] == '-' &&
            argv[i][1] == 'b' &&
            argv[i][2] == 'r')
            {
               convert_br = 1;
            }                 /* end if -br option                   */

         /************************************************************/
         /*                                                          */
         /* -reg option ... convert register instructions            */
         /*                                                          */
         /************************************************************/

         if(argv[i][0] == '-' &&
            argv[i][1] == 'r' &&
            argv[i][2] == 'e' &&
            argv[i][3] == 'g')
            {
               convert_reg = 1;
            }                 /* end if -reg option                  */

         /************************************************************/
         /*                                                          */
         /* -mac option ... convert specific macros                  */
         /*                                                          */
         /************************************************************/

         if(argv[i][0] == '-' &&
            argv[i][1] == 'm' &&
            argv[i][2] == 'a' &&
            argv[i][3] == 'c')
            {
               convert_macro = 1;
            }                 /* end if -mac option                  */

         /************************************************************/
         /*                                                          */
         /* -spm option ... convert structured programming macros    */
         /*                                                          */
         /************************************************************/

         if(argv[i][0] == '-' &&
            argv[i][1] == 's' &&
            argv[i][2] == 'p' &&
            argv[i][3] == 'm')
            {
               convert_spm = 1;
            }                 /* end if -spm option                  */

         /************************************************************/
         /*                                                          */
         /* -half option ... convert halfword instrctions that use   */
         /*                  literals                                */
         /*                                                          */
         /************************************************************/

         if(argv[i][0] == '-' &&
            argv[i][1] == 'h' &&
            argv[i][2] == 'a' &&
            argv[i][3] == 'l' &&
            argv[i][4] == 'f')
            {
               convert_half = 1;
            }                 /* end if -half option                 */

         /************************************************************/
         /*                                                          */
         /* -half31 option ... convert halfword instrctions that use */
         /*                    literals to 31-bit instructions       */
         /*                                                          */
         /************************************************************/

         if(argv[i][0] == '-' &&
            argv[i][1] == 'h' &&
            argv[i][2] == 'a' &&
            argv[i][3] == 'l' &&
            argv[i][4] == 'f' &&
            argv[i][5] == '3' &&
            argv[i][6] == '1')
            {
               convert_half31 = 1;
            }                 /* end if -half31 option               */

         /************************************************************/
         /*                                                          */
         /* -sidcode option .. puts rt halfword instrctions that use */
         /*                    literals to 31-bit instructions       */
         /*                                                          */
         /************************************************************/

         if(argv[i][0] == '-' &&
            argv[i][1] == 's' &&
            argv[i][2] == 'i' &&
            argv[i][3] == 'd' &&
            argv[i][4] == 'c' &&
            argv[i][5] == 'o' &&
            argv[i][6] == 'd' &&
            argv[i][7] == 'e')
            {
               use_sidcode = 1;
               i++;
               k = strlen(&argv[i][0]);
               if(k <= 8)
                  memcpy(&sidcode[0],&argv[i][0],k);
               else
               {
                  memcpy(&sidcode[0], &default_sidcode[0], 8);
                  printf("Sidcode %s is greater than 8 characters.\n",
                         &argv[i][0]);
                }             /* end if else                         */
            }                 /* end if -sidcode                     */

         /************************************************************/
         /*                                                          */
         /* -verify option ... verify file extensions are either:    */
         /*                    .asm, .cpy, or .mac                   */
         /*                                                          */
         /************************************************************/

         if(argv[i][0] == '-' &&
            argv[i][1] == 'v' &&
            argv[i][2] == 'e' &&
            argv[i][3] == 'r' &&
            argv[i][4] == 'i' &&
            argv[i][5] == 'f' &&
            argv[i][6] == 'y')
            {
               verify_input_file_extensions = 1;
            }                 /* end if -verify option               */

      }                       /* end for                             */
   }                          /* end if argc > 4                     */
   else
   {
      /* default                                                     */
      convert_br = 1;         /* convert all branch instr            */
      convert_reg = 1;        /* convert all register type instr     */
      convert_macro = 1;      /* convert all macros                  */
      convert_half = 1;       /* convert halfword instructions       */
      convert_spm = 1;        /* convert SPMs                        */
      convert_half31 = 0;     /* do not use 31-bit instr             */
      use_sidcode = 0;        /* use default sidcode                 */
   }                          /* end if-else argc > 3                */

   if (use_sidcode == 0)
      memcpy(&sidcode[0], &default_sidcode[0], 8);

  /*******************************************************************/
  /* If the request only specifies -sidcode, use default settings.   */
  /*******************************************************************/

   if (convert_br    == 0 &&
       convert_reg   == 0 &&
       convert_macro == 0 &&
       convert_half  == 0 &&
       convert_spm   == 0 &&
       use_sidcode   == 1)
   {
      convert_br = 1;
      convert_reg = 1;
      convert_macro = 1;
      convert_half = 1;
      convert_spm = 1;
   }                          /* end if only -sidcode input          */

   /******************************************************************/
   /*                                                                */
   /*  Verify basics of input parameters                             */
   /*                                                                */
   /*  (1) Something was input for both from_pgm and to_pgm.         */
   /*  (2) from_pgm must have a .asm or .cpy at the end              */
   /*                                                                */
   /******************************************************************/

   h = 1;                    /* Indicate file extension is OK       */
   if (verify_input_file_extensions == 1)
      h = 0;                 /* Indicate '.asm' or '.cpy' not found */
   for(i=0; from_pgm[i] != '\0'; i++)
   {
      if(from_pgm[i] == '.')
         if(from_pgm[i+1] == 'a')
         {
            if(from_pgm[i+2] == 's')
               if(from_pgm[i+3] == 'm')
                  h = 1;     /* Indicate '.asm' found                */
         }        /* end if asm                                      */
         else if(from_pgm[i+1] == 'c')
         {
            if(from_pgm[i+2] == 'p')
               if(from_pgm[i+3] == 'y')
                  h = 1;     /* Indicate '.cpy' found                */
         }        /* end else-if cpy                                 */
         else if(from_pgm[i+1] == 'm')
         {
            if(from_pgm[i+2] == 'a')
               if(from_pgm[i+3] == 'c')
                  h = 1;     /* Indicate '.mac' found                */
         }        /* end else-if cpy                                 */
   }              /* end for                                         */

   printf("Sidcode used to mark changes is %s\n",&sidcode[0]);
   if(convert_reg == 1)
      printf("Register instructions will be converted\n");
   if(convert_br == 1)
      printf("Branch instructions will be converted\n");
   if(convert_macro == 1)
      printf("Macros will be converted\n");
   if(convert_half == 1)
      printf("Halfword instructions will be converted\n");
   if(convert_half31 == 1)
      printf("Halfword instructions will use 31 bit instructions\n");
   if(convert_spm == 1)
      printf("SPMs will be converted\n");

   printf("Input file is %s\n",&from_pgm[0]);
   printf("Output file is %s\n",&to_pgm[0]);

   if(i == 0)
   {
      printf("First argument not input\n");
      return 0;
   }              /* end if i == 0                                   */

   if(h == 0)
   {
      printf("First argument not an assembler file, file extension must be .asm, .cpy, or .mac\n");
      return 0;
   }              /* end if h == 0                                   */

   for(i=0; to_pgm[i] != '\0'; i++);

   if( i == 0)
   {
      printf("Second argument not input\n");
      return 0;
   }              /* end if i == 0                                   */

   /******************************************************************/
   /*                                                                */
   /*  Open the files                                                */
   /*                                                                */
   /******************************************************************/

   from_fp = fopen(from_pgm, "r");
   to_fp = fopen(to_pgm, "w");

   if(from_fp == NULL)
   {
      printf("cannot open %s\n",from_pgm);
      return 0;
   }              /* end if from_fp == NULL                          */

   //printf("DIAGNOSTIC: File %s is open\n", from_pgm);                           //@@DIAG

   rewind(from_fp);

   if(to_fp == NULL)
   {
      printf("cannot open %s\n",to_pgm);
      return 0;
   }              /* end if to_fp == NULL                            */

   //printf("DIAGNOSTIC: File %s is open\n", to_pgm);                             //@@DIAG

   /******************************************************************/
   /*                                                                */
   /*  Do the conversion                                             */
   /*                                                                */
   /******************************************************************/

   lines_converted = 0;
   lines_not_converted = 0;
   lines_to_be_investigated = 0;
   line_nbr = 0;
   first_time_thru = 0;
   previous_line_continued = 0;
   this_line_continued = 0;

   do
   {
      rc = get_source_line(&source_line[0], from_fp);
      line_nbr++;

      //printf("DIAGNOSTIC: Line number %d was read\n", line_nbr);                //@@DIAG

      comment = 0;                 /* source line is not a comment   */
      if(source_line[0] == '*')    /* Is this a comment line?        */
         comment = 1;              /* source line is a comment       */
      else
         if(source_line[0] == '.' && source_line[1] == '*')
            comment = 1;           /* source line is a comment       */

      previous_line_continued = this_line_continued;
      for(i=0; source_line[i] != '\n'; i++);
      if(i >= 71)
         if(source_line[71] != ' ')
            this_line_continued = 1;
         else
            this_line_continued = 0;

                                   /* Determine if the line is all   */         //@FIX-2
                                   /* blanks. If Yes, treat the line */         //@FIX-2
                                   /* like a comment.                */         //@FIX-2
      for(i=0; source_line[i] != '\n' && source_line[i] != ' '; i++);           //@FIX-2
      if(i >= MAXLINE)
         comment = 1;

      if(comment == 0 && previous_line_continued == 0)
      {                            /* No, continue checking          */
         for(i=0;                  /* Get past any labels            */
             source_line[i] != '\n' && source_line[i] != ' ';
             i++);

         label_size = i;
                                   /* Get to OPCODE                  */
         for(; source_line[i] != '\n' && source_line[i] == ' ';  i++);

                                   /**********************************/
                                   /* Make OPCODE all caps           */
                                   /**********************************/
         if(source_line[i] != '\n' && previous_line_continued == 0)
         {                         /* Do OPCODE                      */
            for(k=i; source_line[k] != ' ' && source_line[k] != '\n'; k++)
               if(source_line[k] >= 'a' && source_line[k] <= 'z')
                  source_line[k] += 'A'-'a';
         }                         /* end if source_line[i] != \n    */

         h = 1;                    /* Indicate conversion required   */

         switch(source_line[i])
         {
            case('A'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = A                                  */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == ' ') /* A instruction          */
               {
                  size_of_old_inst = 1;
                  new_inst_ptr = &agf[0];
                  for(m=i+2; source_line[m] != ','; m++);
                  m++;
                  if(source_line[m] == '=') /* 2nd operand a literal */
                  {
                     if (convert_half != 0)
                     {
                        if(validate_literal(&source_line[m]) == 0)
                        {
                           if (convert_half31 != 0)
                              new_inst_ptr = &ahi[0];
                           else
                              new_inst_ptr = &aghi[0];
                           sign = 0;        /* Positive sign         */
                           if(convert_halfword(i, size_of_old_inst, &source_line[0], new_inst_ptr, sign) == 0)
                           {
                              for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                                 source_line[i] = sidcode[m];
                              lines_converted++;
                           } /* end if convert halfword = 0 (worked)    */
                           else
                           {
                              printf("Unable to convert line number %d\n",line_nbr);
                              lines_not_converted++;
                              for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                                 source_line[i] = errorcode[m];
                           } /* end if else convert shift inst = 0 (not done) */
                           h = 0;
                        } /* end if validate_literal == 0            */
                     }    /* end if convert_half != 0                */
                  }    /* end if 2nd operand is a literal            */

                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if A instruction                            */

               /******************************************************/
               /*                                                    */
               /*   Instruction = AH                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'H' &&
                       source_line[i+2] == ' ') /* AH instruction    */
               {
                  for(m=i+3; source_line[m] != ','; m++);
                  m++;
                  if(source_line[m] == '=') /* 2nd operand a literal */
                  {
                     size_of_old_inst = 2;
                     new_inst_ptr = &aghi[0];
                     if (convert_half != 0)
                     {
                        if (convert_half31 != 0)
                           new_inst_ptr = &ahi[0];
                        sign = 0;           /* Positive sign         */
                        if(convert_halfword(i, size_of_old_inst, &source_line[0], new_inst_ptr, sign) == 0)
                        {
                           for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                              source_line[i] = sidcode[m];
                           lines_converted++;
                        } /* end if convert halfword = 0 (worked)    */
                        else
                        {
                           printf("Unable to convert line number %d\n",line_nbr);
                           lines_not_converted++;
                           for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                              source_line[i] = errorcode[m];
                        } /* end if else convert shift inst = 0 (not done) */
                     } /* end if convert_half != 0                   */
                  }    /* end if 2nd operand is a literal            */
                  h = 0;
               }  /* end if AH instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = AL                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                  source_line[i+2] == ' ') /* AL instruction         */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &algf[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if AL instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = AR                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                  source_line[i+2] == ' ') /* AR instruction         */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &agr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if AR instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = AHI                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'H' &&
                       source_line[i+2] == 'I' &&
                       source_line[i+3] == ' ') /* AHI instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &aghi[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if AHI instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = ALR                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == ' ') /* ALR instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &algr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if ALR instruction                          */
               else
                  h = 0;           /* Conversion not done            */
               break;


            case('B'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = B                                  */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == ' ') /* B instruction          */
               {
                  size_of_old_inst = 1;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &j[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if B instruction                            */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BC                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'C' &&
                       source_line[i+2] == ' ') /* BC instruction    */
               {
                  size_of_old_inst = 2;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &brc[0];
                  else
                     h = 0;
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BC instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BE                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'E' &&
                       source_line[i+2] == ' ') /* BE instruction    */
               {
                  size_of_old_inst = 2;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &je[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BE instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BH                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'H' &&
                       source_line[i+2] == ' ') /* BH instruction    */
               {
                  size_of_old_inst = 2;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jh[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BH instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BL                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                       source_line[i+2] == ' ') /* BL instruction    */
               {
                  size_of_old_inst = 2;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jl[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BL instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BM                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'M' &&
                       source_line[i+2] == ' ') /* BM instruction    */
               {
                  size_of_old_inst = 2;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jm[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BM instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BO                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'O' &&
                       source_line[i+2] == ' ') /* BO instruction    */
               {
                  size_of_old_inst = 2;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jo[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BO instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BP                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'P' &&
                       source_line[i+2] == ' ') /* BP instruction    */
               {
                  size_of_old_inst = 2;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jp[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BP instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BZ                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'Z' &&
                       source_line[i+2] == ' ') /* BZ instruction    */
               {
                  size_of_old_inst = 2;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jz[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BZ instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BAL                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'A' &&
                       source_line[i+2] == 'L' &&
                       source_line[i+3] == ' ') /* BAL instruction   */
               {
                  size_of_old_inst = 3;
                  if(is_bal_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &bras[0]; /* Br to label convert */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BAL instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BAS                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'A' &&
                       source_line[i+2] == 'S' &&
                       source_line[i+3] == ' ') /* BAS instruction   */
               {
                  size_of_old_inst = 3;
                  if(is_bal_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &bras[0]; /* Br to label convert */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BAS instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BCT                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'C' &&
                       source_line[i+2] == 'T' &&
                       source_line[i+3] == ' ') /* BCT instruction   */
               {
                  size_of_old_inst = 3;
                  if(convert_br == 0 && convert_reg == 0)
                     h = 0;        /* don't convert                  */
                  else
                  {
                     for(k=i+3; source_line[k] != ' '; k++);
                     for(; source_line[k] != ','; k++);
                     for(m=0; source_line[k] != ' ' && m == 0; k++)
                        if(source_line[k] == '(')
                           m = 1;
                     if(convert_br != 0)
                     {
                        if(m == 0)
                           new_inst_ptr = &brct[0];
                        else if(convert_reg == 0)
                           h = 0;
                     }             /* end if convert_br != 0         */
                     if(convert_reg != 0)
                     {
                        if(m == 0)
                           new_inst_ptr = &brctg[0];
                        else
                           new_inst_ptr = &bctg[0];
                     }             /* end if convert_reg != 0        */
                  }                /* end else                       */
               }  /* end if BCT instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BNE                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'N' &&
                       source_line[i+2] == 'E' &&
                       source_line[i+3] == ' ') /* BNE instruction   */
               {
                  size_of_old_inst = 3;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jne[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BNE instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BNH                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'N' &&
                       source_line[i+2] == 'H' &&
                       source_line[i+3] == ' ') /* BNH instruction   */
               {
                  size_of_old_inst = 3;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jnh[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BNH instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BNL                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'N' &&
                       source_line[i+2] == 'L' &&
                       source_line[i+3] == ' ') /* BNL instruction   */
               {
                  size_of_old_inst = 3;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jnl[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BNL instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BNM                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'N' &&
                       source_line[i+2] == 'M' &&
                       source_line[i+3] == ' ') /* BNM instruction   */
               {
                  size_of_old_inst = 3;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jnm[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BNM instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BNO                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'N' &&
                       source_line[i+2] == 'O' &&
                       source_line[i+3] == ' ') /* BNO instruction   */
               {
                  size_of_old_inst = 3;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jno[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BNO instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BNP                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'N' &&
                       source_line[i+2] == 'P' &&
                       source_line[i+3] == ' ') /* BNP instruction   */
               {
                  size_of_old_inst = 3;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jnp[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BNP instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BNZ                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'N' &&
                       source_line[i+2] == 'Z' &&
                       source_line[i+3] == ' ') /* BNZ instruction   */
               {
                  size_of_old_inst = 3;
                  if(is_branch_to_label(i, size_of_old_inst, &source_line[0]) == 0)
                     new_inst_ptr = &jnz[0];  /* Br to label, convert  */
                  else
                     h = 0;        /* Br to register, don't convert  */
                  if (convert_br == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BNZ instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BXH                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'X' &&
                       source_line[i+2] == 'H' &&
                       source_line[i+3] == ' ') /* BXH instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &brxhg[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BXH instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BCTR                               */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'C' &&
                       source_line[i+2] == 'T' &&
                       source_line[i+3] == 'R' &&
                       source_line[i+4] == ' ') /* BCTR instruction  */
               {
                  size_of_old_inst = 4;
                  new_inst_ptr = &bctgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BCTR instruction                         */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BRCT                               */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'C' &&
                       source_line[i+3] == 'T' &&
                       source_line[i+4] == ' ') /* BRCT instruction  */
               {
                  size_of_old_inst = 4;
                  new_inst_ptr = &brctg[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BRCT instruction                         */

               /******************************************************/
               /*                                                    */
               /*   Instruction = BXLE                               */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'X' &&
                       source_line[i+2] == 'L' &&
                       source_line[i+3] == 'E' &&
                       source_line[i+4] == ' ') /* BXLE instruction  */
               {
                  size_of_old_inst = 4;
                  new_inst_ptr = &brxlg[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if BXLE instruction                         */
               else
                  h = 0;           /* Conversion not done            */
               break;


            case('C'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = C                                  */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == ' ') /* C instruction          */
               {
                  size_of_old_inst = 1;
                  new_inst_ptr = &cgf[0];
                  for(m=i+2; source_line[m] != ','; m++);
                  m++;
                  if(source_line[m] == '=') /* 2nd operand a literal */
                  {
                     x = 0;           /* Indicate convert not done   */
                     if (convert_half != 0)
                     {
                        if(validate_literal(&source_line[m]) == 0)
                        {
                           x = 1;     /* Indicate convert done       */
                           if (convert_half31 != 0)
                              new_inst_ptr = &chi[0];
                           else
                              new_inst_ptr = &cghi[0];
                           sign = 0;        /* Positive sign         */
                           if(convert_halfword(i, size_of_old_inst, &source_line[0], new_inst_ptr, sign) == 0)
                           {
                              for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                                 source_line[i] = sidcode[m];
                              lines_converted++;
                           } /* end if convert halfword = 0 (worked)    */
                           else
                           {
                              printf("Unable to convert line number %d\n",line_nbr);
                              lines_not_converted++;
                              for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                                 source_line[i] = errorcode[m];
                              x = 0;  /* Indicate convert not done   */
                           } /* end if else convert shift inst = 0 (not done) */
                           h = 0;
                        } /* end if validate_literal == 0            */
                     }    /* end if convert_half != 0                */
                     if (x == 0)      /* If halfword immediate       */
                                      /* conversion not done, use    */
                                      /* instruction CLGF rather than*/
                                      /* CGF.                        */
                        new_inst_ptr = &clgf[0];
                  }    /* end if 2nd operand is a literal            */

                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if C instruction                            */

               /******************************************************/
               /*                                                    */
               /*   Instruction = CR                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                  source_line[i+2] == ' ') /* CR instruction         */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &cgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if CR instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = CL                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                  source_line[i+2] == ' ') /* CL instruction         */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &clgf[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if CL instruction                           */


               /******************************************************/
               /*                                                    */
               /*   Instruction = CH                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'H' &&
                       source_line[i+2] == ' ') /* CH instruction    */
               {
                  for(m=i+3; source_line[m] != ','; m++);
                  m++;
                  if(source_line[m] == '=') /* 2nd operand a literal */
                  {
                     size_of_old_inst = 2;
                     new_inst_ptr = &cghi[0];
                     if (convert_half != 0)
                     {
                        if (convert_half31 != 0)
                           new_inst_ptr = &chi[0];
                        sign = 0;           /* Positive sign         */
                        if(convert_halfword(i, size_of_old_inst, &source_line[0], new_inst_ptr, sign) == 0)
                        {
                           for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                              source_line[i] = sidcode[m];
                           lines_converted++;
                        } /* end if convert halfword = 0 (worked)    */
                        else
                        {
                           printf("Unable to convert line number %d\n",line_nbr);
                           lines_not_converted++;
                           for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                              source_line[i] = errorcode[m];
                        } /* end if else convert shift inst = 0 (not done) */
                     } /* end if convert_half != 0                   */
                  }    /* end if 2nd operand is a literal            */
                  h = 0;
               }  /* end if CH instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = CHI                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'H' &&
                       source_line[i+2] == 'I' &&
                       source_line[i+3] == ' ') /* CHI instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &cghi[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if CHI instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = CLR                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == ' ') /* CLR instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &clgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if CLR instruction                          */
               else
                  h = 0;           /* Conversion not done            */
               break;


            case('I'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = ICM                                */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == 'C' &&
                  source_line[i+2] == 'M' &&
                  source_line[i+3] == ' ') /* ICM instruction   */
               {
                  size_of_old_inst = 3;
                  start_of_icm = i;
                  for(i=i+size_of_old_inst; source_line[i] == ' '; i++);
                  for(k=0; source_line[i] != ','; i++, k++)
                     operand[k] = source_line[i];
                  operand[k] = '\0';
                  i++;
                  for(x=i; source_line[x] != ','; x++);
                  x++;
                  for(k=0; source_line[x] != ' '; x++, k++)
                     operand2[k] = source_line[x];
                  operand2[k] = '\0';
                  m = 0;
                  if(source_line[i]   == '1' &&
                          source_line[i+1] == '5' &&
                          source_line[i+2] == ',')
                     m = 1;
                  else if(source_line[i]   == 'B' &&
                          source_line[i+1] == '\'' &&
                          source_line[i+2] == '1' &&
                          source_line[i+3] == '1' &&
                          source_line[i+4] == '1' &&
                          source_line[i+5] == '1' &&
                          source_line[i+6] == '\'' &&
                          source_line[i+7] == ',')
                     m = 1;
                  else if(source_line[i]   == 'b' &&
                          source_line[i+1] == '\'' &&
                          source_line[i+2] == '1' &&
                          source_line[i+3] == '1' &&
                          source_line[i+4] == '1' &&
                          source_line[i+5] == '1' &&
                          source_line[i+6] == '\'' &&
                          source_line[i+7] == ',')
                     m = 1;
                  if (convert_reg == 0)
                     m = 0;        /* skip ICM                       */
                  if(m == 1)
                  {
                     if (output_source_line[0] != '*')
                     {
                        if (output_source_line[0] != '.' &&
                            output_source_line[1] != '*')
                        {
                           /******************************************/
                           /* If an LGHI exists immediately before   */
                           /* the ICM and if the register used in    */
                           /* the LGHI is the same register that is  */
                           /* operand 1 of the ICM, then the LGHI    */
                           /* may have been inserted by a prior run  */
                           /* of this conversion program.  Do not add*/
                           /* LGHI again.  Even if this conversion   */
                           /* program did not insert the LGHI, the   */
                           /* LGHI would accomplish what we want     */
                           /* anyway.                                */
                           /******************************************/
                           for(k=0; output_source_line[k] != ' '; k++);
                           for(;output_source_line[k] == ' '; k++);
                           if(output_source_line[k]   == 'L' &&
                              output_source_line[k+1] == 'G' &&
                              output_source_line[k+2] == 'H' &&
                              output_source_line[k+3] == 'I' &&
                              output_source_line[k+4] == ' ')
                           {
                              for(k+=4; output_source_line[k] == ' '; k++);
                              x = strlen(&operand[0]);
                              if(memcmp(&output_source_line[k], &operand[0], x) == 0)
                                 m = 0;    /* don't convert          */
                           } /* end if output_source_line not LGHI   */
                        }   /* end if output_source_line not comment */
                     }      /* end if output_source_line not comment */

                            /*****************************************/
                            /* If the following is coded:            */
                            /*      ICM  Rx,15,0(Rx)                 */
                            /* where operand 1 and operand 2 use the */
                            /* same register, do not add an LGHI     */
                            /* before ICM.                           */
                            /*****************************************/

                     for(k=i; source_line[k] != ','; k++);
                     for(;source_line[k] != '(' && source_line[k] != ' '; k++);
                     if(source_line[k] == '(')
                     {
                        x = strlen(&operand[0]);
                        if(memcmp(&source_line[k+1], &operand[0], x) == 0)
                           m = 0;          /* don't convert          */
                        if(operand[0] == 'R')
                           if(memcmp(&source_line[k+1], &operand[1], x-1) == 0)
                              m = 0;
                        if(source_line[k+1] == 'R')
                           if(memcmp(&source_line[k+2], &operand[0], x) == 0)
                              m = 0;
                     }      /* end if source_line[k] = '('           */
                  }         /* end if m == 1                         */

                  if(m == 1)
                  {
                     fputs(&output_source_line[0], to_fp);

                     //printf("DIAGNOSTIC: Line number %d was put into file %s at location 1\n", line_nbr-1, to_pgm);        //@@DIAG
                     //printf("DIAGNOSTIC: Line number %d has %d characters\n", line_nbr-1, strlen(&output_source_line[0])); //@@DIAG
                     //printf("DIAGNOSTIC: Line number %d is %s\n", line_nbr-1, &output_source_line[0]);                     //@@DIAG

                     for(k=0; k < MAXLINE; k++)
                        new_source_line[k] = ' ';
                     new_source_line[k] = '\0';

                     for(k=0; source_line[k] != ' '; k++)
                     {
                        new_source_line[k] = source_line[k];
                        source_line[k] = ' ';
                     }             /* end for                        */

                     if (k > 7)
                        k++;
                     else
                        k = 9;

                     /************************************************/
                     /* Change ICM to be 2 instructions:             */
                     /*   LLGF  Rx,operand2                          */
                     /*   LTGR  Rx,Rx                                */
                     /* Do LLGF first:                               */
                     /************************************************/

                     for (m=0; llgf[m] != '\0'; k++, m++)
                        new_source_line[k] = llgf[m];

                     k += 2;

                     for(m=0; operand[m] != '\0'; k++, m++)
                        new_source_line[k] = operand[m];

                     new_source_line[k] = ',';
                     k++;
                     for(m=0; operand2[m] != '\0'; k++, m++)
                        new_source_line[k] = operand2[m];

                     for(m=0, k=72; sidcode[m] != '\0'; k++, m++)
                        new_source_line[k] = sidcode[m];
                     new_source_line[k] = '\n';
                     new_source_line[k+1] = '\0';

                     lines_converted++;
                     strcpy(&output_source_line[0], &new_source_line[0]);

                     /************************************************/
                     /* Do LTGR second:                              */
                     /************************************************/

                     i = start_of_icm;
                     k = i+3;
                     for(; source_line[k] == ' '; k++);
                     source_line[k] = ' ';
                     k++;
                     for(m=0; operand[m] != '\0'; k++, m++)
                        source_line[k] = operand[m];
                     source_line[k] = ',';
                     k++;
                     for(m=0; operand[m] != '\0'; k++, m++)
                        source_line[k] = operand[m];
                     for(; source_line[k] != ' '; k++)
                        source_line[k] = ' ';
                     new_inst_ptr = &ltgr[0];
                  } /* end if m = 1 insert a LLGF before ICM         */
                  else
                   h = 0;
               }  /* end if ICM instruction                          */
               else
                  h = 0;
               break;


            case('L'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = L                                  */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == ' ') /* L instruction          */
               {
                  size_of_old_inst = 1;
                  new_inst_ptr = &llgf[0];
                  for(m=i+2; source_line[m] != ','; m++);
                  m++;
                  if(source_line[m] == '=') /* 2nd operand a literal */
                  {
                     if (convert_half != 0)
                     {
                        if(validate_literal(&source_line[m]) == 0)
                        {
                           if (convert_half31 != 0)
                              new_inst_ptr = &lhi[0];
                           else
                              new_inst_ptr = &lghi[0];
                           sign = 0;        /* Positive sign         */
                           if(convert_halfword(i, size_of_old_inst, &source_line[0], new_inst_ptr, sign) == 0)
                           {
                              for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                                 source_line[i] = sidcode[m];
                              lines_converted++;
                           } /* end if convert halfword = 0 (worked)    */
                           else
                           {
                              printf("Unable to convert line number %d\n",line_nbr);
                              lines_not_converted++;
                              for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                                 source_line[i] = errorcode[m];
                           } /* end if else convert shift inst = 0 (not done) */
                           h = 0;
                        } /* end if validate_literal == 0            */
                     }    /* end if convert_half != 0                */
                  }    /* end if 2nd operand is a literal            */

                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if L instruction                            */

               /******************************************************/
               /*                                                    */
               /*   Instruction = LA                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'A' &&
                       source_line[i+2] == ' ') /* LA instruction    */
               {
                  if (convert_half != 0)
                  {
                     for(k=i+2; source_line[k] == ' '; k++);
                             /* source_line[k] points to 1st operand */
                     h = 1;         /* convert instruction           */
                     for(m=k; source_line[m] != ','; m++);
                     m++;
                     for(x=m; source_line[x] != ' ' && source_line[x] != '('; x++)
                     {
                        if(source_line[x] < '0' || source_line[x] > '9')
                           h = 0; /* if any non-numeric characters, don't convert */
                     }              /* end for                       */
                     if(h == 1)
                     {
                        if(source_line[x] == ' ')
                                /* It has been determined that this  */
                                /* instruction is: LA   Rx,number    */
                                /* Convert to:     LHI  Rx,number    */
                                /*         or:     LGHI Rx,number    */
                        {
                           size_of_old_inst = 2;
                           if (convert_half31 != 0)
                              new_inst_ptr = &lhi[0];
                           else
                              new_inst_ptr = &lghi[0];
                        }           /* end if source_line[x] = ' '   */
                        else
                        {
                                /* at this point, source_line[...]   */
                                /* [i] -> start of opcode            */
                                /* [k] -> start of 1st operand       */
                                /* [m] -> start of 2nd operand       */
                                /* [x] -> (                          */
                           x++;
                           if(source_line[x] == ',')
                              x++;
                           if(memcmp(&source_line[k], &source_line[x], m-k-1) == 0)
                           {
                              x += m-k-1;
                              if(source_line[x] == ')')
                              {
                                /* It has been determined that this  */
                                /* instruction is: LA   Rx,number(Rx)*/
                                /*             or: LA   Rx,number(,Rx)*/
                                /* Convert to:     AHI  Rx,number    */
                                /*         or:     AGHI Rx,number    */
                                 for(x=m; source_line[x] != '('; x++);
                                 for(; source_line[x] != ' '; x++)
                                    source_line[x] = ' ';
                                 size_of_old_inst = 2;
                                 if (convert_half31 != 0)
                                    new_inst_ptr = &ahi[0];
                                 else
                                    new_inst_ptr = &aghi[0];
                              }    /* end if source_line[x] = )      */
                              else
                                 h = 0;    /* don't convert          */
                              }    /* end if memcmp == 0             */
                           else    /* not a LA to be converted       */
                              h = 0;    /* don't convert             */
                        }       /* end else if source_line[x] = ' '  */
                     }              /* end if h = 1                  */
                  }                 /* end if convert_half != 0      */
                  else              /* don't do these conversions    */
                     h = 0;         /* don't convert                 */
               }  /* end if LA instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = LR                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == ' ') /* LR instruction    */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &lgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if LR instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = LH                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'H' &&
                       source_line[i+2] == ' ') /* LH instruction    */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &lgh[0];
                  for(m=i+3; source_line[m] != ','; m++);
                  m++;
                  if(source_line[m] == '=') /* 2nd operand a literal */
                  {
                     if (convert_half != 0)
                     {
                        if (convert_half31 != 0)
                           new_inst_ptr = &lhi[0];
                        else
                           new_inst_ptr = &lghi[0];
                        sign = 0;           /* Positive sign         */
                        if(convert_halfword(i, size_of_old_inst, &source_line[0], new_inst_ptr, sign) == 0)
                        {
                           for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                              source_line[i] = sidcode[m];
                           lines_converted++;
                        } /* end if convert halfword = 0 (worked)    */
                        else
                        {
                           printf("Unable to convert line number %d\n",line_nbr);
                           lines_not_converted++;
                           for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                              source_line[i] = errorcode[m];
                        } /* end if else convert shift inst = 0 (not done) */
                        h = 0;
                     } /* end if convert_half != 0                   */
                  }    /* end if 2nd operand is a literal            */
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if LH instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = LM                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'M' &&
                       source_line[i+2] == ' ') /* LM instruction    */
               {
                  h = 0;           /* don't convert                  */
                  if (convert_reg != 0)
                  {
                     printf("LM instruction was found on line number %d, further investigation is needed\n",line_nbr);
                     for(m=0, i=72; investigatecode[m] != '\0'; m++, i++)
                        source_line[i] = investigatecode[m];
                     lines_to_be_investigated++;
                  }
               }  /* end if LM instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = LCR                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'C' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == ' ') /* LCR instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &lcgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if LCR instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = LHI                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'H' &&
                       source_line[i+2] == 'I' &&
                       source_line[i+3] == ' ') /* LHI instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &lghi[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if LHI instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = LNR                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'N' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == ' ') /* LNR instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &lngr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if LNR instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = LPR                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'P' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == ' ') /* LPR instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &lpgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if LPR instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = LTR                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'T' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == ' ') /* LTR instruction   */
               {
                  for(k=0; output_source_line[k] != ' '; k++);
                  for(;output_source_line[k] == ' '; k++);

                  /***************************************************/
                  /* If the following was coded:                     */
                  /*     L     Rx,label                              */
                  /*     LTR   Rx,Rx                                 */
                  /* It will be converted to:                        */
                  /*     LLGF  Rx,label                              */
                  /*     LTGR  Rx,Rx                                 */
                  /* If a LLGF is the instruction immediately before */
                  /* the LTR instruction, change the LLGF to a LGF   */
                  /* so that the conversion will be:                 */
                  /*     LGF   Rx,label                              */
                  /*     LTGR  Rx,Rx                                 */
                  /***************************************************/

                  if(output_source_line[k]   == 'L' &&
                     output_source_line[k+1] == 'L' &&
                     output_source_line[k+2] == 'G' &&
                     output_source_line[k+3] == 'F' &&
                     output_source_line[k+4] == ' ')
                  {
                     for(x = k+4; output_source_line[x] == ' '; x++);
                     for(y = 0; output_source_line[x+y] != ','; y++);
                     for(m = i+3; source_line[m] == ' '; m++);
                     if(memcmp(&output_source_line[x], &source_line[m], y) == 0)
                     {
                        m = m + y + 1;
                        if(memcmp(&output_source_line[x], &source_line[m], y) == 0)
                        {
                           if(convert_reg != 0)
                           {
                              output_source_line[k+1] = 'G';
                              output_source_line[k+2] = 'F';
                              output_source_line[k+3] = ' ';
                           } /* end if convert_reg is set            */
                        }    /* end if memcmp == 0                   */
                     }       /* end if memcmp 1st operand are equal  */
                  }          /* end if LLGF                          */
                  size_of_old_inst = 3;
                  new_inst_ptr = &ltgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if LTR instruction                          */
               else
                  h = 0;           /* Conversion not done            */
               break;

            case('M'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = MH                                 */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == 'H' &&
                  source_line[i+2] == ' ') /* MH instruction    */
               {
                  for(m=i+3; source_line[m] != ','; m++);
                  m++;
                  if(source_line[m] == '=') /* 2nd operand a literal */
                  {
                     size_of_old_inst = 2;
                     new_inst_ptr = &mghi[0];
                     if (convert_half != 0)
                     {
                        if (convert_half31 != 0)
                           new_inst_ptr = &mhi[0];
                        sign = 0;           /* Positive sign         */
                        if(convert_halfword(i, size_of_old_inst, &source_line[0], new_inst_ptr, sign) == 0)
                        {
                           for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                              source_line[i] = sidcode[m];
                           lines_converted++;
                        } /* end if convert halfword = 0 (worked)    */
                        else
                        {
                           printf("Unable to convert line number %d\n",line_nbr);
                           lines_not_converted++;
                           for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                              source_line[i] = errorcode[m];
                        } /* end if else convert shift inst = 0 (not done) */
                     } /* end if convert_half != 0                   */
                  }    /* end if 2nd operand is a literal            */
                  h = 0;
               }  /* end if MH instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = MHI                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'H' &&
                       source_line[i+2] == 'I' &&
                       source_line[i+3] == ' ') /* MHI instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &mghi[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if MHI instruction                          */

               else
                  h = 0;           /* Conversion not done            */
               break;


            case('N'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = NR                                 */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == 'R' &&
                  source_line[i+2] == ' ') /* NR instruction         */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &ngr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if NR instruction                           */
               else
                  h = 0;           /* Conversion not done            */
               break;


            case('O'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = OR                                 */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == 'R' &&
                  source_line[i+2] == ' ') /* OR instruction         */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &ogr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if OR instruction                           */
               else
                  h = 0;           /* Conversion not done            */
               break;


            case('S'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = S                                  */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == ' ') /* S instruction          */
               {
                  size_of_old_inst = 1;
                  new_inst_ptr = &sgf[0];
                  for(m=i+2; source_line[m] != ','; m++);
                  m++;
                  if(source_line[m] == '=') /* 2nd operand a literal */
                  {
                     if (convert_half != 0)
                     {
                        if(validate_literal(&source_line[m]) == 0)
                        {
                           if (convert_half31 != 0)
                              new_inst_ptr = &ahi[0];
                           else
                              new_inst_ptr = &aghi[0];
                           sign = 1;        /* Negative sign         */
                           if(convert_halfword(i, size_of_old_inst, &source_line[0], new_inst_ptr, sign) == 0)
                           {
                              for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                                 source_line[i] = sidcode[m];
                              lines_converted++;
                           } /* end if convert halfword = 0 (worked)    */
                           else
                           {
                              printf("Unable to convert line number %d\n",line_nbr);
                              lines_not_converted++;
                              for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                                 source_line[i] = errorcode[m];
                           } /* end if else convert shift inst = 0 (not done) */
                           h = 0;
                        } /* end if validate_literal == 0            */
                     }    /* end if convert_half != 0                */
                  }    /* end if 2nd operand is a literal            */

                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if S instruction                            */

               /******************************************************/
               /*                                                    */
               /*   Instruction = SH                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'H' &&
                       source_line[i+2] == ' ') /* SH instruction    */
               {
                  for(m=i+3; source_line[m] != ','; m++);
                  m++;
                  if(source_line[m] == '=') /* 2nd operand a literal */
                  {
                     size_of_old_inst = 2;
                     new_inst_ptr = &aghi[0];
                     if (convert_half != 0)
                     {
                        if (convert_half31 != 0)
                           new_inst_ptr = &ahi[0];
                        sign = 1;           /* Negative sign         */
                        if(convert_halfword(i, size_of_old_inst, &source_line[0], new_inst_ptr, sign) == 0)
                        {
                           for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                              source_line[i] = sidcode[m];
                           lines_converted++;
                        } /* end if convert halfword = 0 (worked)    */
                        else
                        {
                           printf("Unable to convert line number %d\n",line_nbr);
                           lines_not_converted++;
                           for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                              source_line[i] = errorcode[m];
                        } /* end if else convert shift inst = 0 (not done) */
                     } /* end if convert_half != 0                   */
                  }    /* end if 2nd operand is a literal            */
                  h = 0;
               }  /* end if SH instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = SL                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                  source_line[i+2] == ' ') /* SL instruction         */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &slgf[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if SL instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = SR                                 */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                  source_line[i+2] == ' ') /* SR instruction         */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &sgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if SR instruction                           */

               /******************************************************/
               /*                                                    */
               /*   Instruction = SLA                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                       source_line[i+2] == 'A' &&
                       source_line[i+3] == ' ') /* SLA instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &slag[0];
                  if (convert_reg != 0)
                  {
                     if(convert_shift_inst(i, size_of_old_inst, &source_line[0], new_inst_ptr) == 0)
                     {
                        for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                           source_line[i] = sidcode[m];
                        lines_converted++;
                     } /* end if convert shift inst = 0 (worked)        */
                     else
                     {
                        printf("Unable to convert line number %d\n",line_nbr);
                        lines_not_converted++;
                        for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                           source_line[i] = errorcode[m];
                     } /* end if else convert shift inst = 0 (not done) */
                  }    /* end if convert_reg == 0                    */
                  h = 0;
               }  /* end if SLA instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = SLL                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                       source_line[i+2] == 'L' &&
                       source_line[i+3] == ' ') /* SLL instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &sllg[0];
                  if (convert_reg != 0)
                  {
                     if(convert_shift_inst(i, size_of_old_inst, &source_line[0], new_inst_ptr) == 0)
                     {
                        for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                           source_line[i] = sidcode[m];
                        lines_converted++;
                     } /* end if convert shift inst = 0 (worked)        */
                     else
                     {
                        printf("Unable to convert line number %d\n",line_nbr);
                        lines_not_converted++;
                        for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                           source_line[i] = errorcode[m];
                     } /* end if else convert shift inst = 0 (not done) */
                  }    /* end if convert_reg == 0                    */
                  h = 0;
               }  /* end if SLL instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = SLR                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == ' ') /* SLR instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &slgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if SLR instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = SRA                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'A' &&
                       source_line[i+3] == ' ') /* SRA instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &srag[0];
                  if (convert_reg != 0)
                  {
                     if(convert_shift_inst(i, size_of_old_inst, &source_line[0], new_inst_ptr) == 0)
                     {
                        for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                           source_line[i] = sidcode[m];
                        lines_converted++;
                     } /* end if convert shift inst = 0 (worked)        */
                     else
                     {
                        printf("Unable to convert line number %d\n",line_nbr);
                        lines_not_converted++;
                        for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                           source_line[i] = errorcode[m];
                     } /* end if else convert shift inst = 0 (not done) */
                  }    /* end if convert_reg == 0                    */
                  h = 0;
               }  /* end if SRA instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = SRL                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'L' &&
                       source_line[i+3] == ' ') /* SRL instruction   */
               {
                  size_of_old_inst = 3;
                  new_inst_ptr = &srlg[0];
                  if (convert_reg != 0)
                  {
                     if(convert_shift_inst(i, size_of_old_inst, &source_line[0], new_inst_ptr) == 0)
                     {
                        for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                           source_line[i] = sidcode[m];
                        lines_converted++;
                     } /* end if convert shift inst = 0 (worked)        */
                     else
                     {
                        printf("Unable to convert line number %d\n",line_nbr);
                        lines_not_converted++;
                        for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                           source_line[i] = errorcode[m];
                     } /* end if else convert shift inst = 0 (not done) */
                  }    /* end if convert_reg == 0                    */
                  h = 0;
               }  /* end if SRL instruction                          */

               /******************************************************/
               /*                                                    */
               /*   Instruction = STM                                */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'T' &&
                       source_line[i+2] == 'M' &&
                       source_line[i+3] == ' ') /* STM instruction   */
               {
                  h = 0;           /* don't convert                  */
                  if (convert_reg != 0)
                  {
                     printf("STM instruction was found on line number %d, further investigation is needed\n",line_nbr);
                     for(m=0, i=72; investigatecode[m] != '\0'; m++, i++)
                        source_line[i] = investigatecode[m];
                     lines_to_be_investigated++;
                  }
               }  /* end if STM instruction                          */

               else
                  h = 0;           /* Conversion not done            */
               break;


            case('X'):

               /******************************************************/
               /*                                                    */
               /*   Instruction = XR                                 */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == 'R' &&
                  source_line[i+2] == ' ') /* XR instruction         */
               {
                  size_of_old_inst = 2;
                  new_inst_ptr = &xgr[0];
                  if (convert_reg == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if XR instruction                           */
               else
                  h = 0;           /* Conversion not done            */
               break;

            case('$'):

               /******************************************************/
               /*                                                    */
               /*   Macro = $ADPC                                    */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == 'A' &&
                  source_line[i+2] == 'D' &&
                  source_line[i+3] == 'P' &&
                  source_line[i+4] == 'C' &&
                  source_line[i+5] == ' ') /* $ADPC macro            */
               {
                  size_of_old_inst = 5;
                  new_inst_ptr = &adpc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $ADPC instruction                        */

               /******************************************************/
               /*                                                    */
               /*   Macro = $CPUC                                    */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'C' &&
                       source_line[i+2] == 'P' &&
                       source_line[i+3] == 'U' &&
                       source_line[i+4] == 'C' &&
                       source_line[i+5] == ' ') /* $CPUC macro       */
               {
                  size_of_old_inst = 5;
                  new_inst_ptr = &cpuc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $CPUC instruction                        */

               /******************************************************/
               /*                                                    */
               /*   Macro = $CCWBC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'C' &&
                       source_line[i+2] == 'C' &&
                       source_line[i+3] == 'W' &&
                       source_line[i+4] == 'B' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $CCWBC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &ccwbc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $CCWBC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $CCWTC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'C' &&
                       source_line[i+2] == 'C' &&
                       source_line[i+3] == 'W' &&
                       source_line[i+4] == 'T' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $CCWTC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &ccwtc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $CCWTC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $CRISC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'C' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == 'I' &&
                       source_line[i+4] == 'S' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $CRISC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &crisc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $CRISC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $DCOLC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'D' &&
                       source_line[i+2] == 'C' &&
                       source_line[i+3] == 'O' &&
                       source_line[i+4] == 'L' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $DCOLC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &dcolc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $DCOLC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $FINDC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'F' &&
                       source_line[i+2] == 'I' &&
                       source_line[i+3] == 'N' &&
                       source_line[i+4] == 'D' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $FINDC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &findc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $FINDC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $FORKC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'F' &&
                       source_line[i+2] == 'O' &&
                       source_line[i+3] == 'R' &&
                       source_line[i+4] == 'K' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $FORKC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &forkc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $FORKC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $GCFBC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'G' &&
                       source_line[i+2] == 'C' &&
                       source_line[i+3] == 'F' &&
                       source_line[i+4] == 'B' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $GCFBC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &gcfbc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $GCFBC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $GETRC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'G' &&
                       source_line[i+2] == 'E' &&
                       source_line[i+3] == 'T' &&
                       source_line[i+4] == 'R' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $GETRC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &getrc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $GETRC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $GEVAC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'G' &&
                       source_line[i+2] == 'E' &&
                       source_line[i+3] == 'V' &&
                       source_line[i+4] == 'A' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $GEVAC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &gevac[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $GEVAC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $GIOBC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'G' &&
                       source_line[i+2] == 'I' &&
                       source_line[i+3] == 'O' &&
                       source_line[i+4] == 'B' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $GIOBC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &giobc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $GIOBC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $GSVAC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'G' &&
                       source_line[i+2] == 'S' &&
                       source_line[i+3] == 'V' &&
                       source_line[i+4] == 'A' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $GSVAC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &gsvac[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $GSVAC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $LCKRC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                       source_line[i+2] == 'C' &&
                       source_line[i+3] == 'K' &&
                       source_line[i+4] == 'R' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $LCKRC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &lckrc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $LCKRC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $LOCKC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'L' &&
                       source_line[i+2] == 'O' &&
                       source_line[i+3] == 'C' &&
                       source_line[i+4] == 'K' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $LOCKC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &lockc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $LOCKC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $MASKC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'M' &&
                       source_line[i+2] == 'A' &&
                       source_line[i+3] == 'S' &&
                       source_line[i+4] == 'K' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $MASKC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &maskc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $MASKC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $MOVEC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'M' &&
                       source_line[i+2] == 'O' &&
                       source_line[i+3] == 'V' &&
                       source_line[i+4] == 'E' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $MOVEC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &movec[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $MOVEC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $RCFBC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'C' &&
                       source_line[i+3] == 'F' &&
                       source_line[i+4] == 'B' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $RCFBC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &rcfbc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $RCFBC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $RECVC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'E' &&
                       source_line[i+3] == 'C' &&
                       source_line[i+4] == 'V' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $RECVC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &recvc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $RECVC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $RELBC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'E' &&
                       source_line[i+3] == 'L' &&
                       source_line[i+4] == 'B' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $RELBC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &relbc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $RELBC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $RELRC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'E' &&
                       source_line[i+3] == 'L' &&
                       source_line[i+4] == 'R' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $RELRC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &relrc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $RELRC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $RETRC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'E' &&
                       source_line[i+3] == 'T' &&
                       source_line[i+4] == 'R' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $RETRC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &retrc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $RETRC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $RIOBC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'I' &&
                       source_line[i+3] == 'O' &&
                       source_line[i+4] == 'B' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $RIOBC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &riobc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $RIOBC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $RSWBC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'R' &&
                       source_line[i+2] == 'S' &&
                       source_line[i+3] == 'W' &&
                       source_line[i+4] == 'B' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $RSWBC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &rswbc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $RSWBC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $SWSPC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'S' &&
                       source_line[i+2] == 'W' &&
                       source_line[i+3] == 'S' &&
                       source_line[i+4] == 'P' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $SWSPC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &swspc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $SWSPC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $TCPLC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'T' &&
                       source_line[i+2] == 'C' &&
                       source_line[i+3] == 'P' &&
                       source_line[i+4] == 'L' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $TCPLC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &tcplc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $TCPLC instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $ULKRC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'U' &&
                       source_line[i+2] == 'L' &&
                       source_line[i+3] == 'K' &&
                       source_line[i+4] == 'R' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $ULKRC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &ulkrc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $ulkrc instruction                       */

               /******************************************************/
               /*                                                    */
               /*   Macro = $UNLKC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'U' &&
                       source_line[i+2] == 'N' &&
                       source_line[i+3] == 'L' &&
                       source_line[i+4] == 'K' &&
                       source_line[i+5] == 'C' &&
                       source_line[i+6] == ' ') /* $UNLKC macro      */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &unlkc[0];
                  if (convert_macro == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if $UNLKC instruction                       */

               else
                  h = 0;           /* Conversion not done            */
               break;

            case('#'):

               /******************************************************/
               /*                                                    */
               /*   SPM = #CASENTR                                   */
               /*                                                    */
               /******************************************************/

               if(source_line[i+1] == 'C' &&
                  source_line[i+2] == 'A' &&
                  source_line[i+3] == 'S' &&
                  source_line[i+4] == 'E' &&
                  source_line[i+5] == 'N' &&
                  source_line[i+6] == 'T' &&
                  source_line[i+7] == 'R' &&
                  source_line[i+8] == ' ') /* #CASENTR SPM           */
               {
                  size_of_old_inst = 8;
                  new_inst_ptr = &spm_cast[0];
                  if (convert_spm == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if #CASENTR SPM                             */

               /******************************************************/
               /*                                                    */
               /*   SPM = #ECASE                                     */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'E' &&
                       source_line[i+2] == 'C' &&
                       source_line[i+3] == 'A' &&
                       source_line[i+4] == 'S' &&
                       source_line[i+5] == 'E' &&
                       source_line[i+6] == ' ') /* #ECASE SPM        */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &spm_ecas[0];
                  if (convert_spm == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if #ECASE SPM                               */

               /******************************************************/
               /*                                                    */
               /*   SPM = #ELOOP                                     */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'E' &&
                       source_line[i+2] == 'L' &&
                       source_line[i+3] == 'O' &&
                       source_line[i+4] == 'O' &&
                       source_line[i+5] == 'P' &&
                       source_line[i+6] == ' ') /* #ELOOP SPM        */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &spm_elop[0];
                  if (convert_spm == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if #ELOOP SPM                               */

               /******************************************************/
               /*                                                    */
               /*   SPM = #ELSEIF                                    */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'E' &&
                       source_line[i+2] == 'L' &&
                       source_line[i+3] == 'S' &&
                       source_line[i+4] == 'E' &&
                       source_line[i+5] == 'I' &&
                       source_line[i+6] == 'F' &&
                       source_line[i+7] == ' ') /* #ELSEIF SPM      */
               {
                  size_of_old_inst = 7;
                  new_inst_ptr = &spm_elif[0];
                  if (convert_spm == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if #ELSEIF SPM                              */

               /******************************************************/
               /*                                                    */
               /*   SPM = #ESRCH                                     */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'E' &&
                       source_line[i+2] == 'S' &&
                       source_line[i+3] == 'R' &&
                       source_line[i+4] == 'C' &&
                       source_line[i+5] == 'H' &&
                       source_line[i+6] == ' ') /* #ESRCH SPM        */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &spm_edo[0];
                  if (convert_spm == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if #ESRCH SPM                               */

               /******************************************************/
               /*                                                    */
               /*   SPM = #ORELSE                                    */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'O' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == 'E' &&
                       source_line[i+4] == 'L' &&
                       source_line[i+5] == 'S' &&
                       source_line[i+6] == 'E' &&
                       source_line[i+7] == ' ') /* #ORELSE SPM       */
               {
                  size_of_old_inst = 7;
                  new_inst_ptr = &spm_orel[0];
                  if (convert_spm == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if #ORELSE SPM                              */

               /******************************************************/
               /*                                                    */
               /*   SPM = #SRCEX                                     */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'S' &&
                       source_line[i+2] == 'R' &&
                       source_line[i+3] == 'C' &&
                       source_line[i+4] == 'E' &&
                       source_line[i+5] == 'X' &&
                       source_line[i+6] == ' ') /* #SRCEX SPM        */
               {
                  size_of_old_inst = 6;
                  new_inst_ptr = &spm_doex[0];
                  if (convert_spm == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if #SRCEX SPM                               */

               /******************************************************/
               /*                                                    */
               /*   SPM = #STRTSRC                                   */
               /*                                                    */
               /******************************************************/

               else if(source_line[i+1] == 'S' &&
                       source_line[i+2] == 'T' &&
                       source_line[i+3] == 'R' &&
                       source_line[i+4] == 'T' &&
                       source_line[i+5] == 'S' &&
                       source_line[i+6] == 'R' &&
                       source_line[i+7] == 'C' &&
                       source_line[i+8] == ' ') /* #STRTSRC SPM      */
               {
                  size_of_old_inst = 8;
                  new_inst_ptr = &spm_do[0];
                  if (convert_spm == 0)
                     h = 0;        /* don't convert                  */
               }  /* end if #STRTSRC SPM                             */

               else
                  h = 0;           /* Conversion not done            */
               break;

            default:
               h = 0;              /* Converstion not required       */
               break;

         }        /* end switch source_line[i]                       */

         /************************************************************/
         /*                                                          */
         /*  Convert Instruction                                     */
         /*                                                          */
         /************************************************************/

         if(h == 1)                /* if conversion is required      */
         {
            size_of_new_inst = strlen(new_inst_ptr);
            for(k = i + size_of_old_inst; source_line[k] == ' '; k++);
            k = k - i - 1;         /* k = pos of 1st non-blank char  */
                                   /* i = pos of 1st char in instr   */
                                   /* 1 = reserve space for 1 blank  */

            /*********************************************************/
            /*                                                       */
            /* Move in new instruction over old instruction          */
            /*                                                       */
            /*********************************************************/

            if (k >= size_of_new_inst)
            {
               for(m=0; new_inst_ptr[m] != '\0'; m++, i++)
                  source_line[i] = new_inst_ptr[m];

               /* if new instr is smaller than old, pad with blanks  */

               while(m < size_of_old_inst)
               {
                  source_line[i] = ' ';
                  m++;
                  i++;
               }  /* end while m < size_of_old_inst                  */

            }     /* end if k > size_of_new_inst                     */

            /*********************************************************/
            /*                                                       */
            /* Cannot simply move new instruction over old one       */
            /*                                                       */
            /*********************************************************/

            else
            {

               /******************************************************/
               /*                                                    */
               /* There is not enough room between the start of the  */
               /* old instruction and the start of the operand to put*/
               /* the new instruction.  Therefore, start the new     */
               /* instruction before the start of the old one.       */
               /*                                                    */
               /******************************************************/

               if(label_size + size_of_new_inst < i + k)
               {
                  i = i + k - size_of_new_inst;
                  for(m=0; new_inst_ptr[m] != '\0'; m++, i++)
                     source_line[i] = new_inst_ptr[m];
               }  /* end if size_of_new_inst                         */
               else
               {

                  /***************************************************/
                  /*                                                 */
                  /* There is not enough room between the start of   */
                  /* line and the operand to insert the new instr.   */
                  /* Send error message, mark it, and give up.       */
                  /*                                                 */
                  /***************************************************/

                  printf("Unable to convert line number %d\n",line_nbr);
                  lines_not_converted++;
                  for(m=0, i=72; errorcode[m] != '\0'; m++, i++)
                     source_line[i] = errorcode[m];
               }  /* end else-if size_of_new_inst                    */
            }     /* end else-if k > size_of_new_inst                */
            if(source_line[72] != '6')
            {
               for(m=0, i=72; sidcode[m] != '\0'; m++, i++)
                  source_line[i] = sidcode[m];
               lines_converted++;
            }     /* end if source_line[73] |= 6, add sid code       */
         }        /* end if h == 1, conversion required              */

      }           /* end if is a comment, not an instruction         */

      /***************************************************************/
      /*                                                             */
      /* Compress the line by adding a newline character after the   */
      /* last non-blank on the line.                                 */
      /*                                                             */
      /***************************************************************/

      for (i=MAXLINE; i > 0 && source_line[i] == ' '; i--);

      source_line[i+1] = '\n';
      source_line[i+2] = '\0';

      /***************************************************************/
      /*                                                             */
      /* To avoid the case where EOF is the first character on the   */
      /* last line of the file, writing of a line will be delayed.   */
      /* The first time thru the source line will not be written.    */
      /* Instead, the first line will be written when the second     */
      /* line is processed.  Once EOF is obtained, if EOF is the     */
      /* first character on the last line, it will be added to the   */
      /* last character on the previous line.  If the previous or    */
      /* second to last line is full, EOF will be written on its     */
      /* own line.                                                   */
      /*                                                             */
      /* Because of this delay, the actual last line will be written */
      /* after the do-while loop.                                    */
      /*                                                             */
      /***************************************************************/

      if(first_time_thru != 0)
      {
         if (rc != EOF)
         {
            fputs(&output_source_line[0], to_fp);

            //printf("DIAGNOSTIC: Line number %d was put into file %s at location 2\n", line_nbr-1, to_pgm);        //@@DIAG
            //printf("DIAGNOSTIC: Line number %d has %d characters\n", line_nbr-1, strlen(&output_source_line[0])); //@@DIAG
            //printf("DIAGNOSTIC: Line number %d is %s\n", line_nbr-1, &output_source_line[0]);                     //@@DIAG

            strcpy(&output_source_line[0], &source_line[0]);
         }              /* end if rc not EOF                         */
         else
         {
            for(i=0; i <= MAXLINE && source_line[i] == ' '; i++);
            if (source_line[i] != '\n')
               if (source_line[i] != '\0')                 /* @FIX-1 */
               {                                           /* @FIX-1 */
                  fputs(&output_source_line[0], to_fp);

                  //printf("DIAGNOSTIC: Line number %d was put into file %s at location 3\n", line_nbr-1, to_pgm);        //@@DIAG
                  //printf("DIAGNOSTIC: Line number %d has %d characters\n", line_nbr-1, strlen(&output_source_line[0])); //@@DIAG
                  //printf("DIAGNOSTIC: Line number %d is %s\n", line_nbr-1, &output_source_line[0]);                     //@@DIAG

                  strcpy(&output_source_line[0], &source_line[0]);
               }                                           /* @FIX-1 */
         }              /* end if else rc not EOF (or rc = EOF)      */
      }                 /* end if not first time through             */
      else
      {
         first_time_thru = 1;
         strcpy(&output_source_line[0], &source_line[0]);
      }                 /* end if not first time thru (or if first)  */

      //printf("DIAGNOSTIC: Line number %d work is complete\n", line_nbr);        //@@DIAG

   } while(rc != EOF);  /* end do-while not end of file              */

   fputs(&output_source_line[0], to_fp);

   //printf("DIAGNOSTIC: Line number %d was put into file %s at location 4\n", line_nbr, to_pgm);          //@@DIAG
   //printf("DIAGNOSTIC: Line number %d has %d characters\n", line_nbr-1, strlen(&output_source_line[0])); //@@DIAG
   //printf("DIAGNOSTIC: Line number %d is %s\n", line_nbr, &output_source_line[0]);                       //@@DIAG

   /******************************************************************/
   /*                                                                */
   /*  Close the files and exit                                      */
   /*                                                                */
   /******************************************************************/

   fclose(from_fp);

   //printf("DIAGNOSTIC: File %s is closed\n", from_pgm);                         //@@DIAG

   fclose(to_fp);
   printf("Conversion complete\n  %d lines converted\n  %d lines not converted\n  %d lines to be investigated\n  %d total lines\n",
           lines_converted, lines_not_converted, lines_to_be_investigated, line_nbr);

   return 0;
}

/*********************************************************************/
/*                                                                   */
/*  get_source_line()                                                */
/*                                                                   */
/*  Read one line of the input source file.  Be sure that at least   */
/*  80 characters are initialized.  Even if the end of line character*/
/*  is at location 20, fill with blanks to column 80.                */
/*                                                                   */
/*********************************************************************/

int   get_source_line(char *source_line, FILE *fp)
{
   int   c;
   int   i;

   i = 0;

   c = fgetc(fp);

   while (i <= MAXLINE && c != '\n' && c != EOF)
   {
      source_line[i] = c;
      c = fgetc(fp);
      i++;
   }        /* end while                                             */

   for(; i <=MAXLINE; i++)
      source_line[i] = ' ';

   source_line[i] = '\n';                                                       //@FIX-2

   return c;
}

/*********************************************************************/
/*                                                                   */
/*  is_branch_to_label ()                                            */
/*                                                                   */
/*  This function will be called when a branch instruction is found. */
/*  The purpose of this function is to determine if the branch is to */
/*  a regsiter or a label.  For example:                             */
/*                                                                   */
/*     B    RETURN  ...is a branch to a label                        */
/*     B    0(,R7)  ...is a branch to a regsiter                     */
/*                                                                   */
/*  Branch to a label is assumed to be a branch within the program   */
/*  and the branch instruction will be converted to a jump.  Yes,    */
/*  the assumption is not valid.  However, the assembler will take   */
/*  an error when the assumption fails.  If the instruction is a     */
/*  branch to a register, the instruction will not be converted.  In */
/*  this case it is known that the assembly will fail.               */
/*                                                                   */
/*  Return:                                                          */
/*          0 = branch to label                                      */
/*          1 = branch to register                                   */
/*                                                                   */
/*********************************************************************/

int   is_branch_to_label(int start_position, int size_of_instruction, char *source_line)
{
   int   rc;
   int   i;

   i = start_position + size_of_instruction;
   for(; source_line[i] == ' '; i++);

   rc = 0;
   for(; source_line[i] != ' '; i++)
   {
      if (source_line[i] == '(')
         rc = 1;
   }

   return rc;

}

/*********************************************************************/
/*                                                                   */
/*  is_bal_to_label ()                                               */
/*                                                                   */
/*  This function will be called when a BAL/BAS instruction is found.*/
/*  The purpose of this function is to determine if the branch is to */
/*  a regsiter or a label.  For example:                             */
/*                                                                   */
/*     BAL  Rx,RETURN  ...is a branch to a label                     */
/*     BAL  Rx,0(,R7)  ...is a branch to a regsiter                  */
/*                                                                   */
/*  BAL to a label is assumed to be a branch within the program      */
/*  and the branch instruction will be converted to a jump.  Yes,    */
/*  the assumption is not valid.  However, the assembler will take   */
/*  an error when the assumption fails.  If the instruction is a     */
/*  branch to a register, the instruction will not be converted.  In */
/*  this case it is known that the assembly will fail.               */
/*                                                                   */
/*  Return:                                                          */
/*          0 = branch to label                                      */
/*          1 = branch to register                                   */
/*                                                                   */
/*********************************************************************/

int   is_bal_to_label(int start_position, int size_of_instruction, char *source_line)
{
   int   rc;
   int   i;

   i = start_position + size_of_instruction;
   for(; source_line[i] == ' '; i++);

   for(; source_line[i] != ','; i++);

   rc = 0;
   for(; source_line[i] != ' '; i++)
   {
      if (source_line[i] == '(')
         rc = 1;
   }

   return rc;

}

/*********************************************************************/
/*                                                                   */
/*  convert_shift_inst ()                                            */
/*                                                                   */
/*  Shift instructions such as SLL need more changes than simply     */
/*  replacing the op code.  For example:                             */
/*                                                                   */
/*    31-bit instr ...   SLL   R1,4                                  */
/*    64-bit instr ...   SLLG  R1,R1,4                               */
/*                                                                   */
/*  This function will convert the instruction as well as adding     */
/*  an additional operand.  The added operand (operand 3) will be the*/
/*  register as operand (1).                                         */
/*                                                                   */
/*  Return:                                                          */
/*          0 = line converted                                       */
/*          1 = line not converted                                   */
/*                                                                   */
/*********************************************************************/

int convert_shift_inst(int start_position, int size_of_inst, char *source_line, char *new_inst)
{
   int   i;
   int   k;
   int   label_size;
   int   rc;
   int   start_of_operand;
   int   size_of_area_needed;
   int   size_of_new_inst;
   int   size_of_operand;
   char  operand[MAXLINE];

   for(label_size=0;
       source_line[label_size] != '\n' && source_line[label_size] != ' ';
       label_size++);

   label_size++;             /* Account for blank following label    */

   i = start_position + size_of_inst;
   for(; source_line[i] == ' '; i++);
   start_of_operand = i;

   for(k = 0; source_line[i] != ','; i++, k++)
      operand[k] = source_line[i];
   operand[k] = '\0';

   size_of_new_inst = strlen(new_inst);
   size_of_operand = strlen(operand);
   size_of_area_needed = size_of_new_inst + size_of_operand + 3;
                             /* +3 = Space before opcode             */
                             /*    + Space between opcode and operand*/
                             /*    + comma (,) after 1st operand     */

   if(start_of_operand - label_size > size_of_area_needed)
   {
      i = start_of_operand - size_of_area_needed + label_size;
      for(k = 0; k < size_of_new_inst; k++, i++)
         source_line[i] = new_inst[k];

      source_line[i] = ' ';
      i++;

      for(k = 0; k < size_of_operand; k++, i++)
         source_line[i] = operand[k];

      source_line[i] = ',';
      rc = 0;
   }               /* end if operand > size of area needed           */
   else
   {
      rc = 1;      /* line not converted                             */
   }

   return rc;
}

/*********************************************************************/
/*                                                                   */
/*  convert_halfword ()                                              */
/*                                                                   */
/*  Halfword instructions will need more changes than replacing the  */
/*  op code.  For example:                                           */
/*                                                                   */
/*    LH    R2,=Y(64)    will be changed to                          */
/*    LGHI  R2,64                                                    */
/*                                                                   */
/*  This function will convert the instruction as well as removing   */
/*  the literal.                                                     */
/*                                                                   */
/*  If sign = 1, then a negative will be inserted before the 2nd     */
/*  operand. This is intended for SH instructions.  There is no SGHI */
/*  or SHI instruction.  Instead an AGHI of AHI will be used with a  */
/*  negative value.  For example:                                    */
/*                                                                   */
/*    SH    R2,=Y(64)    will be changed to                          */
/*    AGHI  R2,-64                                                   */
/*                                                                   */
/*  Return:                                                          */
/*          0 = line converted                                       */
/*          1 = line not converted                                   */
/*                                                                   */
/*********************************************************************/

int convert_halfword(int start_position, int size_of_inst, char *source_line, char *new_inst, int sign)
{
   int   i;
   int   k;
   int   label_size;
   int   paren;
   int   rc;
   int   start_of_operand;
   int   size_of_area_needed;
   int   size_of_new_inst;

   for(label_size=0;
       source_line[label_size] != '\n' && source_line[label_size] != ' ';
       label_size++);

   i = start_position + size_of_inst;
   for(; source_line[i] == ' '; i++);
   start_of_operand = i;

   size_of_new_inst = strlen(new_inst);
   size_of_area_needed = size_of_new_inst + 2;

   if(start_of_operand - label_size >= size_of_area_needed)
   {
      if(start_position + size_of_new_inst +1 < start_of_operand)
         i = start_position;
      else
         i = start_of_operand - size_of_new_inst - 1;

      for(k = 0; k < size_of_new_inst; k++, i++)
         source_line[i] = new_inst[k];  /* copy new instruction      */

      source_line[i] = ' ';

      for(i = start_of_operand;
          source_line[i] != ',';
          i++);              /* Go to end of 1st operand             */

      i++;                   /* Go past ,                            */

      if(sign == 1)          /* Add negative sign?                   */
      {
         source_line[i] = '-';
         i++;
      }                      /* end if sign = 1                      */

      for(k = i;
          source_line[k] != '(' && source_line[k] != '\'' && source_line[k] != '\n';
          k++);              /* Go past =y( or =al2( or =h' or =xl2( */

      if(source_line[k] == '(')
      {
         paren = 1;          /* Literal uses ( )                     */
         if(sign == 1)       /* Negative signed added?               */
         {                   /* Yes                                  */
            source_line[i] = source_line[k]; /* Copy the (           */
            i++;
         }                   /* end if sign = 1                      */
      }                      /* end if source_line[k] = (            */
      else
         paren = 0;          /* Literal uses ' '                     */

      k++;                   /* Go past the ( or '                   */

      if(paren == 1)
      {
         for(; paren >= 1; k++, i++)
         {
            source_line[i] = source_line[k]; /* copy value to instr  */
            if (source_line[k] == '(')
               paren++;
            else if (source_line[k] == ')')
               paren--;
            else if (source_line[k] == '\n')
               paren = 0;
         }                   /* end for                              */
         if (sign != 1)      /* Negative sign used?                  */
            i--;             /* No, negative sign, reset index       */
                             /* Yes, nagative sign, include )        */
      }                      /* end if paren = 1                     */
      else
      {
         for(; source_line[k] != '\'' && source_line[k] != '\n'; k++, i++)
            source_line[i] = source_line[k]; /* copy value to instr     */
      }                      /* end if else paren = 1                */

      for(; k > i; i++)
         source_line[i] = ' '; /* overlay remaining value with blanks */

      for(; source_line[i] != ' ' && source_line[i] != '\n'; i++)
         source_line[i] = ' '; /* overlay remaining literal with blanks */

      rc = 0;
   }               /* end if operand > size of area needed           */
   else
   {
      rc = 1;      /* line not converted                             */
   }

   return rc;
}

/*********************************************************************/
/*                                                                   */
/*  valid_literal()                                                  */
/*                                                                   */
/*  Determine if this literal can be converted into an immediate     */
/*  instructions.  For example:                                      */
/*                                                                   */
/*    A     R2,=F'100'   can be changed to                           */
/*    AGHI  R2,100                                                   */
/*                                                                   */
/*  This function will only accept at literals which are numerical   */
/*  values (e.g. =A(200) or =F'100')                                 */
/*                                                                   */
/*  Input:                                                           */
/*    char *literal  is a pointer to the start of the literal        */
/*                                                                   */
/*  Return:                                                          */
/*          0 = valid literal                                        */
/*          1 = not valid, don't convert this instruction            */
/*                                                                   */
/*********************************************************************/

int validate_literal( char *literal)
{

#define MAXVALUE 32768

   int   i;
   int   rc;
   int   value;

   rc = 1;                             /* Indicate not valid         */
   value = 0;

   if(literal[0] == '=')               /* Is this a literal?         */
   {
      if (literal[1] != 'B' &&
          literal[1] != 'b' &&
          literal[1] != 'C' &&
          literal[1] != 'c' &&
          literal[1] != 'X' &&
          literal[1] != 'x')
      {
         for(i=1;
             literal[i] != '(' && literal[i] != '\'' && literal[i] != '\n';
             i++);

         i++;                          /* Move past literal          */

         for(; isdigit(literal[i]); i++)
            value = (10 * value) + (literal[i] - '0');

         if(literal[i] == ')' || literal[i] == '\'')
            if (value < MAXVALUE)
               rc = 0;
      }                           /* end if literal != B or C or X  */
   }
                                  /* end if this is a literal?  */
   return rc;
}
